@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: makemeta.c 8588 2009-06-02 01:48:48Z charles $
+ * $Id: makemeta.c 8720 2009-06-20 00:39:30Z charles $
  */
 
 #include <assert.h>
@@ -296,13 +296,18 @@ getFileInfo( const char *                     topFile,
              tr_benc *                        uninitialized_path )
 {
     const char * pch, *prev;
-    const size_t topLen = strlen( topFile ) + 1; /* +1 for '/' */
+    size_t       topLen;
     int          n;
 
     /* get the file size */
     tr_bencInitInt( uninitialized_length, file->size );
 
-    /* the path list */
+    /* how much of file->filename to walk past */
+    topLen = strlen( topFile );
+    if( topLen>0 && topFile[topLen-1]!=TR_PATH_DELIMITER )
+        ++topLen; /* +1 for the path delimiter */
+
+    /* build the path list */
     n = 1;
     for( pch = file->filename + topLen; *pch; ++pch )
         if( *pch == TR_PATH_DELIMITER )
