@@ -3,11 +3,11 @@
  *
  * This file is licensed by the GPL version 2.  Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
- * so that the bulk of its code can remain under the MIT license. 
+ * so that the bulk of its code can remain under the MIT license.
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: torrent.h 8709 2009-06-16 19:31:17Z charles $
+ * $Id: torrent.h 8915 2009-08-13 17:50:41Z charles $
  */
 
 #ifndef __TRANSMISSION__
@@ -20,7 +20,7 @@
 #include "completion.h" /* tr_completion */
 #include "ratecontrol.h" /* tr_ratecontrol */
 #include "session.h" /* tr_globalLock(), tr_globalUnlock() */
-#include "utils.h" /* tr_bitfield */
+#include "utils.h" /* TR_GNUC_PRINTF */
 
 struct tr_bandwidth;
 struct tr_ratecontrol;
@@ -118,6 +118,11 @@ tr_torrent*      tr_torrentNext( tr_session  * session,
 
 void             tr_torrentCheckSeedRatio( tr_torrent * tor );
 
+/** save a torrent's .resume file if it's changed since the last time it was saved */
+void             tr_torrentSave( tr_torrent * tor );
+
+void             tr_torrentSetLocalError( tr_torrent * tor, const char * fmt, ... ) TR_GNUC_PRINTF( 2, 3 );
+
 
 
 typedef enum
@@ -140,7 +145,7 @@ struct tr_torrent
 
     int                      magicNumber;
 
-    int                      error;
+    tr_stat_errtype          error;
     char                     errorString[128];
 
     uint8_t                  obfuscatedHash[SHA_DIGEST_LENGTH];
@@ -185,6 +190,9 @@ struct tr_torrent
     uint64_t                   corruptCur;
     uint64_t                   corruptPrev;
 
+    uint64_t                   etaSpeedCalculatedAt;
+    double                     etaSpeed;
+
     time_t                     addedDate;
     time_t                     activityDate;
     time_t                     doneDate;
@@ -201,6 +209,7 @@ struct tr_torrent
     tr_bool                    isDeleting;
     tr_bool                    needsSeedRatioCheck;
     tr_bool                    startAfterVerify;
+    tr_bool                    isDirty;
 
     uint16_t                   maxConnectedPeers;
 
@@ -219,6 +228,8 @@ struct tr_torrent
 
     double                     desiredRatio;
     tr_ratiolimit              ratioLimitMode;
+
+    uint64_t                   preVerifyTotal;
 };
 
 /* get the index of this piece's first block */
@@ -288,15 +299,20 @@ static TR_INLINE tr_bool tr_torrentIsPrivate( const tr_torrent * tor )
 
 static TR_INLINE tr_bool tr_torrentAllowsPex( const tr_torrent * tor )
 {
-    return ( tor != NULL  ) && tor->session->isPexEnabled && !tr_torrentIsPrivate( tor );
+    return ( tor != NULL )
+        && ( tor->session->isPexEnabled )
+        && ( !tr_torrentIsPrivate( tor ) );
 }
 
 static TR_INLINE tr_bool tr_torrentAllowsDHT( const tr_torrent * tor )
 {
-    return ( tor != NULL  ) && tor->session->isDHTEnabled && !tr_torrentIsPrivate( tor );
+    return ( tor != NULL )
+        && ( tr_sessionAllowsDHT( tor->session ) )
+        && ( !tr_torrentIsPrivate( tor ) );
 }
 
-static TR_INLINE tr_bool tr_torrentIsPieceChecked( const tr_torrent  * tor, tr_piece_index_t i )
+static TR_INLINE tr_bool tr_torrentIsPieceChecked( const tr_torrent  * tor,
+                                                   tr_piece_index_t    i )
 {
     return tr_bitfieldHasFast( &tor->checkedPieces, i );
 }
@@ -317,4 +333,13 @@ static inline tr_bool tr_isTorrent( const tr_torrent * tor )
         && ( tr_isSession( tor->session ) );
 }
 
+/* set a flag indicating that the torrent's .resume file
+ * needs to be saved when the torrent is closed */
+static inline void tr_torrentSetDirty( tr_torrent * tor )
+{
+    assert( tr_isTorrent( tor ) );
+
+    tor->isDirty = TRUE;
+}
+
 #endif
